var _ioService=Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);


function Plugin() {
   this.id = null;
   this.owner = null;
   this.entryEditors = {};
   this.entryToolbarItems = {};
}

var _Plugin_NAMESPACE = "http://www.atomojo.org/Vocabulary/Atomic/Plugin/2007/1/0";

Plugin.prototype.load = function(manifest) {
   this.id = manifest.getAttribute("id");
   var current = this;
   var baseURI = _ioService.newURI(manifest.baseURI,null,null);
   this.owner = DOMUtil.textOf(manifest,_Plugin_NAMESPACE,"owner");
   DOMUtil.forChild(manifest,_Plugin_NAMESPACE,"entry-editor",function(target) {
      var editor = {
         id: target.getAttribute("id"),
         label: DOMUtil.textOf(target,_Plugin_NAMESPACE,"label"),
         description: DOMUtil.textOf(target,_Plugin_NAMESPACE,"description")
      }
      DOMUtil.forChild(target,_Plugin_NAMESPACE,"editor",function(link) {
         var url = _ioService.newURI(link.getAttribute("href"),null,baseURI).spec;
         editor["xul"] = url;
      });
      DOMUtil.forChild(target,_Plugin_NAMESPACE,"icon",function(link) {
         var url = _ioService.newURI(link.getAttribute("src"),null,baseURI).spec;
         editor["icon"] = url;
      });
      current.entryEditors[editor.id] = editor;
   });
   DOMUtil.forChild(manifest,_Plugin_NAMESPACE,"toolbar",function(toolbar) {
      if (toolbar.getAttribute("id")!="entry-toolbar") {
         alert("Unsupported toolbar "+toolbar.getAttribute("id"));
         return;
      }
      DOMUtil.forChild(toolbar,_Plugin_NAMESPACE,"entry-editor",function(link) {
         var id = link.getAttribute("ref");
         current.entryToolbarItems[id] = current.entryEditors[id];
      });
   });
   DOMUtil.forChild(manifest,_Plugin_NAMESPACE,"action",function(target) {
      if (target.getAttribute("type")!="edit") {
         return;
      }
      if (target.getAttribute("target")!="entry") {
         return;
      }
      DOMUtil.forChild(target,_Plugin_NAMESPACE,"association",function(link) {
         var id = link.getAttribute("to");
         var editor = current.entryEditors[id];
         var matcher = new EntryMatcher();
         DOMUtil.forChild(link,ATOM_NS,"category",function(category) {
            var scheme = link.getAttribute("scheme");
            var term = link.getAttribute("term");
            matcher.categories[matcher.categories.length] = {
               scheme: scheme,
               term: term
            };
         });
      });
   });
}

function EntryMatcher() {
   this.categories = [];
   this.editor = null;
}

EntryMatcher.prototype.match = function(entry) {
   var matches = [];
   for (var i=0; i<this.categories.length; i++) {
      matches[i] = false;
   }
   var current = this;
   DOMUtil.forChild(entry.xml,ATOM_NS,"category",function(c) {
      var scheme = link.getAttribute("scheme");
      var term = link.getAttribute("term");
      for (var i=0; i<current.categories.length; i++) {
         if (current.categories[i].term==term && current.categories[i].scheme==scheme) {
            matches[i] = true;
         }
      }
   });
   for (var i=0; i<this.categories.length; i++) {
      if (!matches[i]) {
         return false;
      }
   }
   return true;
}

